//
// Genius Scan SDK
//
// Copyright 2010-2019 The Grizzly Labs
//
// Subject to the Genius Scan SDK Licensing Agreement
// sdk@thegrizzlylabs.com
//

#import <CoreMedia/CoreMedia.h>
#import <UIKit/UIKit.h>

#import <GSSDK/GSKPDFGeneratorError.h>

NS_ASSUME_NONNULL_BEGIN

@class GSKPDFDocument;
@class GSKPDFGeneratorConfiguration;
@class GSKQuadrangle;
@class GSKTextLayout;

@protocol GSKPDFImageProcessor;
@protocol GSKLogger;

/**
 Error handling
 */
FOUNDATION_EXPORT NSString * const GSKErrorDomain;
typedef NS_ENUM(NSInteger, GSKError) {
    /// An license key related error.
    GSKLicenseError = 1000,
    /// An internal error. Please report it to sdk@geniusscan.com with output logs.
    GSKInternalError = 1001,
    /// An image processing error
    GSKProcessingError = 1002,
    /// A general error due to a bad input parameter. Verify that your input match the API requirements.
    GSKBadInputError = 1003,
    /// A file not found error
    GSKFileNotFound = 1004,
};

FOUNDATION_EXPORT NSString * const GSKPDFErrorDomain;

/**
 The different possible enhancements
 */
typedef NS_ENUM(NSInteger, GSKFilterType) {
    /// No post processing
    GSKFilterNone = 0,
    /**
     The black and white enhancement results in a mostly black and white image but gray levels
     are used for antialiasing. The background is turned white.
     */
    GSKFilterBlackAndWhite,
    /// Turns the background white but preserves the color.
    GSKFilterColor,
    /// Enhancement to apply to photographs.
    GSKFilterPhoto,
    /// Monochrome enhancement. The resulting image contains only two colors.
    GSKFilterMonochrome,
};

/**
 Quadrangle detection options
 */
typedef NS_OPTIONS(NSInteger, GSKDetectQuadrangleOptions) {
    /// No options
    GSKDetectQuadrangleOptionsNone = 0,
    /// Request a fast quadrangle detection (useful for real-time detection)
    GSKDetectQuadrangleOptionsFast = 1 << 0,
};

/**
 The result of the quadrangle detection operation.
 */
@interface GSKQuadrangleDetectionResult: NSObject

- (instancetype)initWithQuadrangle:(GSKQuadrangle *)quadrangle;

/**
 The detected quadrangle or null if not quadrangle has been detected
 */
@property (nonatomic, strong, nullable, readonly) GSKQuadrangle *quadrangle;

@end

@interface GSKBestFilterResult: NSObject

/**
 The detected best post-processing to apply to the image.
 */
@property (nonatomic, assign, readonly) GSKFilterType filterType;

@end

/**
 The SDK class is your entry point to the low-level SDK features.
*/
@interface GSK : NSObject

/**
 Must be called first with a valide license key to enable
 the SDK.

 @warning *Important* This method must be called before any use of the SDK methods.

 All the SDK errors return proper errors in that case and you can handle them to ensure you provide a good "degraded" experience. For instance, you can prompt the user to update the application to use the scanning feature in case they use a version of the application with an expired license key.

 @param licenseKey The license key
 @param error On input, a pointer to an error object. If an error occurs, this pointer is set to an actual error object containing the error information. You may specify nil for this parameter if you do not want the error information.

 @return YES if the SDK was initialized successfully, NO if an error occured.
 */
+ (BOOL)initWithLicenseKey:(NSString *)licenseKey error:(NSError * _Nullable *)error;

/**
 Detects the quadrangle corresponding to the edges of a document in a photo

 @param image The photo to detect a document in
 @param options See GSKDetectQuadrangleOptions
 @param error On input, a pointer to an error object. If an error occurs, this pointer is set to an actual error object containing the error information. You may specify nil for this parameter if you do not want the error information.

 @return The detected quadrangle or nil if an error occured.
 */
+ (GSKQuadrangleDetectionResult * _Nullable)detectQuadrangleFromImage:(UIImage *)image options:(GSKDetectQuadrangleOptions)options error:(NSError * _Nullable *)error;

/**
 Detects the quadrangle corresponding to the edges of a document in video frame

 @param sampleBuffer a YCbCr sample buffer of a video frame. Make sure the video output generating these frames is configured with the pixel format kCVPixelFormatType_420YpCbCr8BiPlanarFullRange
 @param error On input, a pointer to an error object. If an error occurs, this pointer is set to an actual error object containing the error information. You may specify nil for this parameter if you do not want the error information.

 @return The detected quadrangle or nil if an error occured.
 */
+ (GSKQuadrangleDetectionResult * _Nullable)detectQuadrangleFromSampleBuffer:(CMSampleBufferRef)sampleBuffer error:(NSError * _Nullable *)error;

/**
 Extracts the document from the given photo and corrects its perspective
 
 @param image The original photo
 @param quadrangle The quadrangle representing the edges of the document. If nil, the entire image is returned.
 @param error On input, a pointer to an error object. If an error occurs, this pointer is set to an actual error object containing the error information. You may specify nil for this parameter if you do not want the error information.

 @return the cropped and perspective-corrected document or nil if an error occured.
         Errors include: - non-convex quadrangle
                         - failure to warp
 */
+ (UIImage * _Nullable)warpImage:(UIImage *)image withQuadrangle:(GSKQuadrangle * _Nullable)quadrangle error:(NSError * _Nullable *)error;

/**
 Determines the best image processing type for a document
 
 @param image The cropped and perspective-corrected document
 @param error On input, a pointer to an error object. If an error occurs, this pointer is set to an actual error object containing the error information. You may specify nil for this parameter if you do not want the error information.

 @return The best enhancement for this document or nil if an error occured.
 */
+ (GSKBestFilterResult * _Nullable)bestFilterForImage:(UIImage *)image error:(NSError * _Nullable *)error;

/**
 Enhances a document image with the selected post processing type
 
 @param image the warped, perspective-corrected document
 @param filter the enhancement filter to apply to the image. @see GSKFilterType
 @param error On input, a pointer to an error object. If an error occurs, this pointer is set to an actual error object containing the error information. You may specify nil for this parameter if you do not want the error information.

 @return the enhanced image or nil if an error occured.
 */
+ (UIImage * _Nullable)enhanceImage:(UIImage *)image withFilter:(GSKFilterType)filter error:(NSError * _Nullable *)error;

/**
 Corrects curvature distortion in the given image. This typically corrects curvature
 seen when scanning a book or when a receipt is folded.

 This method is meant to be applied on a warped image.

 @param image the input image.
 @param error On input, a pointer to an error object. If an error occurs, this pointer is set to an actual error object containing the error information. You may specify nil for this parameter if you do not want the error information.

 @return the corrected image or nil if an error occured.
 */
+ (UIImage * _Nullable)correctImageDistortion:(UIImage *)image error:(NSError * _Nullable *)error;

@end

/**
 GSKPDF encapsulate the generation of a PDF document.
 */
@interface GSKPDF : NSObject

/**
 Write the given PDF document to the specified file path.

 @param document The PDF document.
 @param filePath The file path to write the PDF to.
 @param processor An object to preprocess each image before creating the corresponding page. For instance, you can inject a preprocessor to downsample the images.
 @param error On input, a pointer to an error object. If an error occurs, this pointer is set to an actual error object containing the error information. You may specify nil for this parameter if you do not want the error information.

 @return YES if the PDF was generated successfully, NO if an error occured.

 This method will take some time, proportionally to the number of pages of the PDF document.
 */
+ (BOOL)generatePDFDocument:(GSKPDFDocument *)document
                     toPath:(NSString *)filePath
         withImageProcessor:(_Nullable id<GSKPDFImageProcessor>)processor
                      error:(NSError * _Nullable *)error;

/**
 A convenience shorthand for the previous method with a processor that doesn't apply any change to the images.
 */
+ (BOOL)generatePDFDocument:(GSKPDFDocument *)document
                     toPath:(NSString *)filePath
                      error:(NSError * _Nullable *)error;

@end

NS_ASSUME_NONNULL_END
