//
//  GSKUIScannerUI.h
//  GSSDKScannerUI
//
//  Created by Bruno Virlet on 4/26/19.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

FOUNDATION_EXPORT NSErrorDomain const GSKScannerUIErrorDomain;

typedef NS_ENUM(NSInteger, GSKScannerUIError) {
    /// User has canceled the scan flow
    GSKScannerUIUserCancelationError = 999,
    /// A general error due to a bad input parameter. Verify that your input match the API requirements.
    GSKScannerUIBadInputError = 1000,
};


@class GSKScannerUIScan;

/**
 The source for a scan flow
 */
typedef NS_ENUM(NSUInteger, GSKScannerUISource) {
    /** Scan from the camera */
    GSKScannerUISourceCamera,
    /** Import a file from an URL */
    GSKScannerUISourceImageURL,
    /** Import a file from the library */
    GSKScannerUISourceLibrary,
};

/**
 The different possible filters for the scanner UI.

 The possible values match the GSKFilterType values except "automatic".
 */
typedef NS_ENUM(NSInteger, GSKScannerUIFilterType) {
    GSKScannerUIFilterAutomatic = 0,
    GSKScannerUIFilterNone,
    GSKScannerUIFilterBlackAndWhite,
    GSKScannerUIFilterColor,
    GSKScannerUIFilterPhoto,
    GSKScannerUIFilterMonochrome,
};

typedef NS_OPTIONS(NSUInteger, GSKScannerUIPostProcessingActions) {
    GSKScannerUIPostProcessingActionNone = 0,
    GSKScannerUIPostProcessingActionEditFilter = 1 << 0,
    GSKScannerUIPostProcessingActionRotate = 1 << 1,
    GSKScannerUIPostProcessingActionAll = NSUIntegerMax,
};

typedef NS_ENUM(NSUInteger, GSKScannerUIPDFPageSize) {
    /// PDF page with a width of 8.27 inch (A4 width) and fitting height.
    GSKScannerUIPDFPageFit,
    GSKScannerUIPDFPageA4,
    GSKScannerUIPDFPageLetter,
};

typedef NS_ENUM(NSUInteger, GSKScannerUIFlashMode) {
    GSKScannerUIFlashModeAuto,
    GSKScannerUIFlashModeOn,
    GSKScannerUIFlashModeOff,
};


/**
 Enables configuration of the GSKUIScanner scan flow.

 Set the desired configuration on an instance of GSKScannerUIConfiguration and pass it to the initializer of GSKUIScanner to build the desired scan flow.
 */
@interface GSKScannerUIConfiguration : NSObject

/**
 Specifies the source for the scanned documents (eg. the camera, …).

 If source is GSKScannerUISourceImageURL, you must provide the sourceImageURL parameter.
 */
@property (nonatomic, assign) GSKScannerUISource source;

/**
 The image URL if source is GSKScannerUISourceImageURL.
 */
@property (nonatomic, strong) NSURL *sourceImageURL;

/**
 Multipage mode. Only apply when the source is GSKScannerUISourceCamera.

 When the multipage mode is active, the user will have the opportunity to snap multiple scans in a row and the output document will contain all these pages.

 When multipage mode is off, the user will only scan a single page. The output document will always contain a single page.

 Defaults to YES.
 */
@property (nonatomic, assign) BOOL multiPage;

/**
 Force the use of a specific post processing.

 Defaults to GSKFilterAutomatic.
 */
@property (nonatomic, assign) GSKScannerUIFilterType defaultFilter;

/**
 Select which actions are displayed in the post processing screen

 Default is everything.
 */
@property (nonatomic, assign) GSKScannerUIPostProcessingActions postProcessingActions;

/**
 Determines the size of the pages when generating the PDF.

 Default is GSKScannerUIPDFPageFit
 */
@property (nonatomic, assign) GSKScannerUIPDFPageSize pdfPageSize;

/**
 Determines the visibility of the flash button
 */
@property (nonatomic, assign) BOOL flashButtonHidden;

/**
 Determines the default flash mode.

 Combine with flashButtonHidden if you want to only allow a specific
 flash mode.

 Defaults to GSKScannerUIFlashModeAuto
 */
@property (nonatomic, assign) GSKScannerUIFlashMode defaultFlashMode;

/**
 UI background color.

 Default is black.
 */
@property (nonatomic, copy) UIColor *backgroundColor;

/**
 UI foreground color (text and icons)

 Default is white.
 */
@property (nonatomic, copy) UIColor *foregroundColor;

/**
 UI highlight color (document detection overlay)

 Default is dark blue.
 */
@property (nonatomic, copy) UIColor *highlightColor;

/**
 UI menu color

 Default is system's default (eg. blue on iOS)
 */
@property (nonatomic, copy) UIColor *menuColor;

/**
 JPEG quality used to compress captured images. Between 0 and 100, 100 being the best quality.
 
 Default is 60
 */
@property (nonatomic, assign) NSInteger jpegQuality;

/**
 Max scan dimension in pixels when generating the PDF file. All scans included in the PDF file
 will be scaled so that both their height and width are smaller than this value, while keeping
 their aspect ratio.
 
 Default is 0, which means no scaling is applied.
 */
@property (nonatomic, assign) NSInteger pdfMaxScanDimension;

@end

/**
 The result of a scan flow
 */
@interface GSKScannerUIResult : NSObject

/**
 The PDF generated at the end of the scan flow
 */
@property (nonatomic, strong) NSURL *pdfURL;

/**
 The individual scans taken during the scan flow.

 Each scan object contains both the original and the enhanced scans.
 */
@property (nonatomic, strong) NSArray<GSKScannerUIScan *> *scans;
@end

/**
 A high-level scanner module.

 You present it and when the user is done, you obtain a result object containing the scanned documents.

 The scan flow can be customized with a GSKScannerUIConfiguration.
 */
@interface GSKScannerUI : NSObject

/**
 Instantiates a GSKUIScanner with the provided configuration.

 @param configuration The provided configuration
 */
+ (instancetype)scannerUIWithConfiguration:(GSKScannerUIConfiguration *)configuration;

/**
 Present the scan flow.

 @param viewController The view controller to present the scan flow from.
 @param completionBlock A completion block that will be called on the main thread upon completion of the scan flow. It received a GSKScannerUIResult object as a parameter. This object gives you access to the scanned documents.
 @param errorBlock A block called upon error
 */
- (void)startFromViewController:(UIViewController *)viewController onSuccess:(void (^_Nonnull)(GSKScannerUIResult *))completionBlock failure:(void (^_Nonnull)(NSError *))errorBlock;

@end

NS_ASSUME_NONNULL_END
